<?php

/*********************************************************************

 PHP Blog Ver 3.01 (2012/12/17)

*********************************************************************/

/********* ライセンス ************************************************

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 @Link http://www.php-labo.net/
 @Copyright(C) 2007-2012 PHP Labo
 @Author Knight <info at php-labo dot net>

*********************************************************************/

/********* 外部ファイル読み込み *************************************/

require_once './smarty/Smarty.class.php';
require_once './config.php';

if (PICTOGRAM_MODE) {
	require_once PICTOGRAM_MAIN_DIR . 'HTML/Emoji.php';
}

/********* メイン処理 ***********************************************/

//ユーザー情報取得
$users = user();

//セッション開始
session();

//データ正規化
normalize();

//設定読み込み
$configs = config();

//URLルーティング
$params = routing();

//データベース接続
$pdo = database();

//テンプレート設定
$smarty = template();

//プラグイン実行
plugin('page');

//モードに応じた処理を実行
switch ($_REQUEST['mode']) {
	case 'setup':
		setup();
		break;
	case 'view':
		view();
		break;
	case 'comment':
		comment();
		break;
	case 'trackback':
		trackback();
		break;
	case 'archive':
		archive();
		break;
	case 'search':
		search();
		break;
	case 'rss':
		rss();
		break;
	case 'admin':
		if (isloggedin()) {
			switch ($_REQUEST['work']) {
				case 'article':
					admin_article();
					break;
				case 'article_form':
					admin_article_form();
					break;
				case 'article_post':
					admin_article_post();
					break;
				case 'article_del':
					admin_article_del();
					break;
				case 'file':
					admin_file();
					break;
				case 'file_post':
					admin_file_post();
					break;
				case 'file_del':
					admin_file_del();
					break;
				case 'comment':
					admin_comment();
					break;
				case 'comment_del':
					admin_comment_del();
					break;
				case 'trackback':
					admin_trackback();
					break;
				case 'trackback_del':
					admin_trackback_del();
					break;
				case 'config':
					admin_config();
					break;
				case 'config_post':
					admin_config_post();
					break;
				case 'logout':
					admin_logout();
					break;
				default:
					admin_status();
			}
		} else {
			admin();
		}
		break;
	default:
		article();
}

exit;

/********* ユーザー定義関数 *****************************************/

/* セットアップ */
function setup()
{
	global $users, $configs, $params, $pdo, $smarty;

	//記事格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'articles(id INT UNSIGNED NOT NULL AUTO_INCREMENT, stat INT UNSIGNED NOT NULL, break INT UNSIGNED NOT NULL, comt INT UNSIGNED NOT NULL, tb INT UNSIGNED NOT NULL, date DATETIME NOT NULL, subj VARCHAR(255) NOT NULL, text TEXT NOT NULL, tag VARCHAR(255), PRIMARY KEY(id))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'articles(id INTEGER, stat INTEGER UNSIGNED NOT NULL, break INTEGER UNSIGNED NOT NULL, comt INTEGER UNSIGNED NOT NULL, tb INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, subj VARCHAR NOT NULL, text TEXT NOT NULL, tag VARCHAR, PRIMARY KEY(id))');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//コメント格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'comments(id INT UNSIGNED NOT NULL AUTO_INCREMENT, article_id INT UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR(80) NOT NULL, mail VARCHAR(80), url VARCHAR(255), text TEXT NOT NULL, host VARCHAR(80), PRIMARY KEY(id))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'comments(id INTEGER, article_id INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR NOT NULL, mail VARCHAR, url VARCHAR, text TEXT NOT NULL, host VARCHAR, PRIMARY KEY(id))');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//トラックバック格納テーブル作成
	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'trackbacks(id INT UNSIGNED NOT NULL AUTO_INCREMENT, article_id INT UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR(80) NOT NULL, title VARCHAR(255) NOT NULL, url VARCHAR(255) NOT NULL, text TEXT, host VARCHAR(80), PRIMARY KEY(id))');
	} else {
		$stmt = $pdo->query('CREATE TABLE ' . DATABASE_PREFIX . 'trackbacks(id INTEGER, article_id INTEGER UNSIGNED NOT NULL, date DATETIME NOT NULL, name VARCHAR NOT NULL, title VARCHAR NOT NULL, url VARCHAR NOT NULL, text TEXT, host VARCHAR, PRIMARY KEY(id))');
	}
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	//完了画面表示
	echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
	echo "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\" dir=\"ltr\">\n";
	echo "<head>\n";
	echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
	echo "<title>Complete</title>\n";
	echo "</head>\n";
	echo "<body>\n";
	echo "<p>セットアップが完了しました。</p>\n";
	echo "</body>\n";
	echo "</html>\n";

	return;
}

/* 記事一覧 */
function article()
{
	global $users, $configs, $params, $pdo, $smarty;

	//表示ページ数取得
	if (isset($params[0])) {
		$_GET['page'] = $params[0];
	}
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//プラグイン実行
	plugin('begin');

	//表示記事番号取得
	$stmt = $pdo->prepare('SELECT id FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 ORDER BY date DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['page_size']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['page_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$shows = array();
	while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$shows[] = $data[0];
	}

	//コメント数取得
	$comments = array();
	if (!empty($shows)) {
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'comments WHERE article_id IN(' . implode(',', $shows) . ') GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$comments[$data[0]] = $data[1];
		}
	}

	//トラックバック数取得
	$trackbacks = array();
	if (!empty($shows)) {
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks WHERE article_id IN(' . implode(',', $shows) . ') GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$trackbacks[$data[0]] = $data[1];
		}
	}

	//記事取得
	if (!empty($shows)) {
		$stmt = $pdo->query('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE id IN(' . implode(',', $shows) . ') ORDER BY date DESC');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}
			$data['tag']       = create_tag($data['tag']);
			$data['comment']   = isset($comments[$data['id']])   ? $comments[$data['id']]   : 0;
			$data['trackback'] = isset($trackbacks[$data['id']]) ? $trackbacks[$data['id']] : 0;

			$articles[] = $data;
		}
		$smarty->assign('articles', $articles);
	}

	//記事数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data          = $stmt->fetch(PDO::FETCH_NUM);
	$article_count = $data[0];
	$article_page  = ceil($article_count / $configs['_blog']['page_size']);

	$smarty->assign('article_count', $article_count);
	$smarty->assign('article_page',  $article_page);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('article.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 記事表示 */
function view()
{
	global $users, $configs, $params, $pdo, $smarty;

	//記事番号取得
	if (isset($params[1])) {
		$_GET['id'] = $params[1];
	}
	if (!isset($_GET['id']) or !preg_match('/^\d+$/', $_GET['id'])) {
		$_GET['id'] = 0;
	}

	if (!$_GET['id']) {
		error('不正なアクセスです。');
	}

	//プラグイン実行
	plugin('begin');

	//コメント数取得
	$stmt = $pdo->prepare('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'comments WHERE article_id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$data    = $stmt->fetch(PDO::FETCH_NUM);
	$comment = $data[0];

	//トラックバック数取得
	$stmt = $pdo->prepare('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks WHERE article_id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$data      = $stmt->fetch(PDO::FETCH_NUM);
	$trackback = $data[0];

	//記事取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND id = :id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		if ($data['break']) {
			$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
		}
		$data['tag']       = create_tag($data['tag']);
		$data['comment']   = $comment;
		$data['trackback'] = $trackback;

		$smarty->assign('article', $data);
	} else {
		error('指定された記事は存在しません。');
	}

	//コメント取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'comments WHERE article_id = :id ORDER BY id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$comments = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$comments[] = $data;
	}
	$smarty->assign('comments', $comments);

	//トラックバック取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'trackbacks WHERE article_id = :id ORDER BY id');
	$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$trackbacks = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$trackbacks[] = $data;
	}
	$smarty->assign('trackbacks', $trackbacks);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('view.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* コメント投稿 */
function comment()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//コメント受付チェック
	$stmt = $pdo->prepare('SELECT comt FROM ' . DATABASE_PREFIX . 'articles WHERE id = :id');
	$stmt->bindValue(':id', $_POST['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$comt = $data[0];

		if (!$comt) {
			error('この記事はコメントを受け付けていません。');
		}
	} else {
		error('指定された記事は存在しません。');
	}

	//入力内容チェック
	if ($_POST['name'] == '') {
		error('名前が入力されていません。');
	}
	if ($_POST['text'] == '') {
		error('本文が入力されていません。');
	}
	if ($_POST['url'] == 'http://') {
		$_POST['url'] = '';
	}

	if (mb_strlen($_POST['name'], 'UTF-8') > 30) {
		error('名前は30文字以内で入力してください。');
	}
	if (mb_strlen($_POST['mail'], 'UTF-8') > 80) {
		error('メールアドレスは80文字以内で入力してください。');
	}
	if (mb_strlen($_POST['url'], 'UTF-8') > 200) {
		error('URLは200文字以内で入力してください。');
	}
	if (mb_strlen($_POST['text'], 'UTF-8') > 1000) {
		error('本文は1000文字以内で入力してください。');
	}

	if (!empty($_POST['mail']) and !strpos($_POST['mail'], '@')) {
		error('メールアドレスの入力内容が正しくありません。');
	}

	//プラグイン実行
	plugin('begin');

	//Cookie登録
	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_URL);
	} else {
		$info = parse_url(HTTP_URL);
	}

	if (isset($_POST['save'])) {
		setcookie('comment[name]', trim($_POST['name']), time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[mail]', trim($_POST['mail']), time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[url]',  trim($_POST['url']),  time() + COOKIE_EXPIRE, $info['path']);
		setcookie('comment[save]', trim($_POST['save']), time() + COOKIE_EXPIRE, $info['path']);
	} else {
		setcookie('comment[name]', '', 0, $info['path']);
		setcookie('comment[mail]', '', 0, $info['path']);
		setcookie('comment[url]',  '', 0, $info['path']);
		setcookie('comment[save]', '', 0, $info['path']);
	}

	//データ投稿
	if (($mail = trim($_POST['mail'])) == '') {
		$mail = null;
	}
	if (($url = trim($_POST['url'])) == '') {
		$url = null;
	}
	if (($host = gethostbyaddr($_SERVER['REMOTE_ADDR'])) == '') {
		$host = null;
	}
	$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_PREFIX . 'comments VALUES(NULL, :id, :date, :name, :mail, :url, :text, :host)');
	$stmt->bindValue(':id',   $_POST['id'], PDO::PARAM_INT);
	$stmt->bindValue(':date', date('Y-m-d H:i:s'));
	$stmt->bindValue(':name', trim($_POST['name']));
	$stmt->bindValue(':mail', $mail);
	$stmt->bindValue(':url',  $url);
	$stmt->bindValue(':text', trim($_POST['text']));
	$stmt->bindValue(':host', $host);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	//プラグイン実行
	plugin('end');

	//リダイレクト
	if (REWRITE_MODE) {
		$script_url = preg_replace('/\/$/', '', HTTP_URL);
	} else {
		$script_url = HTTP_URL . MAIN_FILE;
	}

	header('Location: ' . $script_url . '/view/' . intval($_POST['id']) . (($users['type'] == 'mobile') ? '?' . session_name() . '=' . session_id() : ''));

	exit;
}

/* トラックバック受信 */
function trackback()
{
	global $users, $configs, $params, $pdo, $smarty;

	//トラックバック受付チェック
	$stmt = $pdo->prepare('SELECT tb FROM ' . DATABASE_PREFIX . 'articles WHERE id = :id');
	$stmt->bindValue(':id', $params[1], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_NUM)) {
		$tb = $data[0];

		if (!$tb) {
			tb_error('Forbidden Post');
		}
	} else {
		tb_error('No Article');
	}

	//受信内容チェック
	if (empty($params[1])) {
		tb_error('No TrackBack ID (tb_id)');
	}
	if (!isset($_POST['blog_name']) or $_POST['blog_name'] == '') {
		tb_error('No Name (blog_name)');
	}
	if (!isset($_POST['title']) or $_POST['title'] == '') {
		tb_error('No Title (title)');
	}
	if (!isset($_POST['url']) or $_POST['url'] == '') {
		tb_error('No URL (url)');
	}

	//プラグイン実行
	plugin('begin');

	//データ投稿
	if (($text = trim(mb_convert_encoding($_POST['excerpt'], 'UTF-8', 'auto'))) == '') {
		$text = null;
	}
	if (($host = gethostbyaddr($_SERVER['REMOTE_ADDR'])) == '') {
		$host = null;
	}
	$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_PREFIX . 'trackbacks VALUES(NULL, :id, :date, :name, :title, :url, :text, :host)');
	$stmt->bindValue(':id',    $params[1], PDO::PARAM_INT);
	$stmt->bindValue(':date',  date('Y-m-d H:i:s'));
	$stmt->bindValue(':name',  trim(mb_convert_encoding($_POST['blog_name'], 'UTF-8', 'auto')));
	$stmt->bindValue(':title', trim(mb_convert_encoding($_POST['title'], 'UTF-8', 'auto')));
	$stmt->bindValue(':url',   trim($_POST['url']));
	$stmt->bindValue(':text',  $text);
	$stmt->bindValue(':host',  $host);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	header('Content-Type: text/xml');

	echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n";
	echo "<response>\n";
	echo "<error>0</error>\n";
	echo "</response>\n";

	//プラグイン実行
	plugin('end');

	return;
}

/* 過去ログ */
function archive()
{
	global $users, $configs, $params, $pdo, $smarty;

	//表示年月日取得
	if (isset($params[1])) {
		$_GET['date'] = $params[1];
	}
	if (!isset($_GET['date']) or !preg_match('/^\d+$/', $_GET['date'])) {
		$_GET['date'] = '';
	}

	//プラグイン実行
	plugin('begin');

	if ($_GET['date']) {
		//コメント数取得
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'comments GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$comments = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$comments[$data[0]] = $data[1];
		}

		//トラックバック数取得
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$trackbacks = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$trackbacks[$data[0]] = $data[1];
		}

		//記事取得
		if (preg_match('/^(\d\d\d\d)$/', $_GET['date'], $matches)) {
			$smarty->assign('view', array(
				'flag' => true,
				'year' => $matches[1]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND DATE_FORMAT(date, \'%Y\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND STRFTIME(\'%Y\', date) = :date ORDER BY date DESC');
			}
		} elseif (preg_match('/^(\d\d\d\d)(\d\d)$/', $_GET['date'], $matches)) {
			$smarty->assign('view', array(
				'flag'  => true,
				'year'  => $matches[1],
				'month' => $matches[2]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND DATE_FORMAT(date, \'%Y%m\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND STRFTIME(\'%Y%m\', date) = :date ORDER BY date DESC');
			}
		} elseif (preg_match('/^(\d\d\d\d)(\d\d)(\d\d)$/', $_GET['date'], $matches)) {
			$smarty->assign('view', array(
				'flag'  => true,
				'year'  => $matches[1],
				'month' => $matches[2],
				'day'   => $matches[3]
			));

			if (DATABASE_TYPE == 'mysql') {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND DATE_FORMAT(date, \'%Y%m%d\') = :date ORDER BY date DESC');
			} else {
				$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND STRFTIME(\'%Y%m%d\', date) = :date ORDER BY date DESC');
			}
		} else {
			error('不正なアクセスです。');
		}
		$stmt->bindValue(':date', $_GET['date']);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}
			$data['tag']       = create_tag($data['tag']);
			$data['comment']   = isset($comments[$data['id']])   ? $comments[$data['id']]   : 0;
			$data['trackback'] = isset($trackbacks[$data['id']]) ? $trackbacks[$data['id']] : 0;

			$articles[] = $data;
		}
		$smarty->assign('articles', $articles);
	} else {
		//月一覧取得
		if (DATABASE_TYPE == 'mysql') {
			$stmt = $pdo->query('SELECT DATE_FORMAT(date, \'%Y-%m\') AS month FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 GROUP BY month ORDER BY month DESC');
		} else {
			$stmt = $pdo->query('SELECT STRFTIME(\'%Y-%m\', date) AS month FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 GROUP BY month ORDER BY month DESC');
		}
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$dates = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if (preg_match('/^(\d\d\d\d)\-(\d\d)$/', $data['month'], $matches)) {
				$year   = $matches[1];
				$month  = $matches[2];
			} else {
				exit('投稿日時の形式が不正です。');
			}

			$dates[] = array(
				'year'  => $year,
				'month' => $month,
			);
		}
		$smarty->assign('dates', $dates);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('archive.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* ログ検索 */
function search()
{
	global $users, $configs, $params, $pdo, $smarty;

	//検索対象取得
	if (isset($params[1])) {
		$_GET['target'] = $params[1];
	}
	if (!isset($_GET['target'])) {
		$_GET['target'] = '';
	}

	//検索語句取得
	if (isset($params[2])) {
		$_GET['word'] = preg_replace('/\+/', ' ', $params[2]);
	}
	if (!isset($_GET['word'])) {
		$_GET['word'] = '';
	}

	//プラグイン実行
	plugin('begin');

	if ($_GET['word'] != '') {
		//コメント数取得
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'comments GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$comments = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$comments[$data[0]] = $data[1];
		}

		//トラックバック数取得
		$stmt = $pdo->query('SELECT article_id, COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks GROUP BY article_id');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$trackbacks = array();
		while ($data = $stmt->fetch(PDO::FETCH_NUM)) {
			$trackbacks[$data[0]] = $data[1];
		}

		//検索結果取得
		if ($_GET['target'] == 'tag') {
			$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND (tag = :word1 OR tag LIKE :word2 OR tag LIKE :word3 OR tag LIKE :word4) ORDER BY date DESC');
			$stmt->bindValue(':word1', $_GET['word']);
			$stmt->bindValue(':word2', $_GET['word'] . ',%');
			$stmt->bindValue(':word3', '%,' . $_GET['word']);
			$stmt->bindValue(':word4', '%,' . $_GET['word'] . ',%');
		} else {
			$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 AND (subj LIKE :word1 OR text LIKE :word2 OR tag LIKE :word3) ORDER BY date DESC');
			$stmt->bindValue(':word1', '%' . $_GET['word'] . '%');
			$stmt->bindValue(':word2', '%' . $_GET['word'] . '%');
			$stmt->bindValue(':word3', '%' . $_GET['word'] . '%');
		}
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$articles = array();
		while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			if ($data['break']) {
				$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
			}
			$data['tag']       = create_tag($data['tag']);
			$data['comment']   = isset($comments[$data['id']])   ? $comments[$data['id']]   : 0;
			$data['trackback'] = isset($trackbacks[$data['id']]) ? $trackbacks[$data['id']] : 0;

			$articles[] = $data;
		}
		$smarty->assign('articles', $articles);
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('search.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* RSS配信 */
function rss()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//記事取得
	$stmt = $pdo->query('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 ORDER BY date DESC LIMIT ' . $configs['_blog']['rss_size']);
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$articles = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		if ($data['break']) {
			$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
		}
		$data['description'] = strip_tags($data['text']);
		$data['description'] = strlen($data['description']) > 255 ? mb_strimwidth($data['description'], 0, 252, '...', 'UTF-8') : $data['description'];

		$articles[] = $data;
	}
	$smarty->assign('articles', $articles);

	//最終更新日時取得
	$stmt = $pdo->query('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE stat = 1 ORDER BY date DESC LIMIT 1');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$smarty->assign('last', array(
			'date' => $data['date']
		));
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('rss.xml');

	//プラグイン実行
	plugin('end');

	return;
}

/* ログイン状態チェック */
function isloggedin()
{
	global $users, $configs, $params, $pdo, $smarty;

	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_URL);
	} else {
		$info = parse_url(HTTP_URL);
	}

	if ($users['serial'] and ADMIN_SERIAL) {
		//固体識別情報確認
		if (in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
			return true;
		}
	} elseif (isset($_POST['password'])) {
		//パスワード認証
		if ($_POST['password'] == ADMIN_PASSWORD) {
			$_SESSION['login'] = true;

			if (isset($_POST['session'])) {
				$session = md5(uniqid(rand(), true));

				if (touch(LOGIN_DIR . $session)) {
					chmod(LOGIN_DIR . $session, 0644);
				} else {
					exit('ログイン情報を保持できません。');
				}

				setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
			}
		}

		header('Location: ' . (HTTPS_URL ? HTTPS_URL : HTTP_URL) . MAIN_FILE . '?mode=admin&work=default' . (($users['type'] == 'mobile') ? '&' . session_name() . '=' . session_id() : ''));

		exit;
	} elseif (empty($_SESSION['login'])) {
		//ログイン状態を復元
		if (isset($_COOKIE['session'])) {
			if ($dir = scandir(LOGIN_DIR)) {
				foreach ($dir as $entry) {
					if (is_file(LOGIN_DIR . $entry) and time() - filemtime(LOGIN_DIR . $entry) > COOKIE_EXPIRE) {
						unlink(LOGIN_DIR . $entry);
					}
				}
			} else {
				exit('ログイン情報格納ディレクトリを開けません。');
			}

			if (file_exists(LOGIN_DIR . $_COOKIE['session'])) {
				$_SESSION['login'] = true;

				$session = md5(uniqid(rand(), true));

				if (rename(LOGIN_DIR . $_COOKIE['session'], LOGIN_DIR . $session) and touch(LOGIN_DIR . $session)) {
					setcookie('session', $session, time() + COOKIE_EXPIRE, $info['path']);
				} else {
					exit('ログイン情報を保持できません。');
				}
			}
		}
	}

	if (isset($_SESSION['login']) and $_SESSION['login'] == true) {
		return true;
	} else {
		return false;
	}
}

/* 管理者用 - 認証画面 */
function admin()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//エラーメッセージ取得
	if ($_REQUEST['work'] == 'default') {
		$smarty->append('errors', 'パスワードが違います。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 記事一覧 */
function admin_article()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//表示ページ取得
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//プラグイン実行
	plugin('begin');

	//記事取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles ORDER BY date DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$articles = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		if ($data['break']) {
			$data['text'] = '<p>' . nl2br($data['text']) . '</p>';
		}
		$data['description'] = strip_tags($data['text']);
		$data['description'] = strlen($data['description']) > 255 ? mb_strimwidth($data['description'], 0, 252, '...', 'UTF-8') : $data['description'];
		$data['tag']         = create_tag($data['tag']);

		$articles[] = $data;
	}
	$smarty->assign('articles', $articles);

	//記事数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'articles');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data          = $stmt->fetch(PDO::FETCH_NUM);
	$article_count = $data[0];
	$article_page  = ceil($article_count / $configs['_blog']['admin_size']);

	$smarty->assign('article_count', $article_count);
	$smarty->assign('article_page',  $article_page);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_article.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 投稿画面 */
function admin_article_form()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//プラグイン実行
	plugin('begin');

	//フォームデータ取得
	if (isset($_POST['work']) and $_POST['work'] == 'article_post') {
		$form_id    = $_POST['id'];
		$form_stat  = $_POST['stat'];
		$form_break = $_POST['break'];
		$form_comt  = $_POST['comt'];
		$form_tb    = $_POST['tb'];
		$form_date  = $_POST['date'];
		$form_subj  = $_POST['subj'];
		$form_text  = $_POST['text'];
		$form_tag   = $_POST['tag'];
	} elseif (isset($_GET['id'])) {
		$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'articles WHERE id = :id');
		$stmt->bindValue(':id', $_GET['id'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		if ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
			$form_id    = $data['id'];
			$form_stat  = $data['stat'];
			$form_break = $data['break'];
			$form_comt  = $data['comt'];
			$form_tb    = $data['tb'];
			$form_date  = $data['date'];
			$form_subj  = $data['subj'];
			$form_text  = $data['text'];
			$form_tag   = $data['tag'];
		} else {
			error('指定された記事は存在しません。');
		}
	} else {
		$form_id    = '';
		$form_stat  = $configs['_blog']['default_stat']  ? true : false;
		$form_break = $configs['_blog']['default_break'] ? true : false;
		$form_comt  = $configs['_blog']['default_comt']  ? true : false;
		$form_tb    = $configs['_blog']['default_tb']    ? true : false;
		$form_date  = date('Y-m-d H:i:00');
		$form_subj  = '';
		$form_text  = '';
		$form_tag   = '';
	}

	$smarty->assign('form', array(
		'id'    => $form_id,
		'stat'  => $form_stat,
		'break' => $form_break,
		'comt'  => $form_comt,
		'tb'    => $form_tb,
		'date'  => $form_date,
		'subj'  => $form_subj,
		'text'  => $form_text,
		'tag'   => $form_tag
	));

	//アップロードファイル取得
	if ($dir = scandir(FILE_DIR)) {
		$files = array();
		foreach ($dir as $entry) {
			if ($entry == '.' or $entry == '..') {
				continue;
			}

			if (preg_match('/^[\w\-]+\.(gif|jpeg|jpg|jpe|png)$/i', $entry)) {
				list($file_width, $file_height) = getimagesize(FILE_DIR . $entry);
				$value = '<img src="' . HTTP_URL . FILE_DIR . $entry . '" alt="' . $entry . '" width="' . $file_width . '" height="' . $file_height . '" />';
			} else {
				$value = '<a href="' . HTTP_URL . FILE_DIR . $entry . '">' . $entry . '</a>';
			}

			$files[] = array(
				'name'  => $entry,
				'value' => $value
			);
		}

		$smarty->assign('files', $files);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_article_form.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 記事投稿 */
function admin_article_post()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!isset($_POST['preview']) and !token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (!isset($_POST['stat'])) {
		$_POST['stat'] = 0;
	}
	if (!isset($_POST['break'])) {
		$_POST['break'] = 0;
	}
	if (!isset($_POST['comt'])) {
		$_POST['comt'] = 0;
	}
	if (!isset($_POST['tb'])) {
		$_POST['tb'] = 0;
	}
	if ($_POST['tag'] != '') {
		$_POST['tag'] = preg_replace('/\s*,\s*/', ',', $_POST['tag']);
	}

	if ($_POST['subj'] == '') {
		$smarty->append('errors', '題名が入力されていません。');
	}
	if ($_POST['text'] == '') {
		$smarty->append('errors', '本文が入力されていません。');
	}

	if (mb_strlen($_POST['subj'], 'UTF-8') > 30) {
		$smarty->append('errors', '題名は30文字以内で入力してください。');
	}
	if (mb_strlen($_POST['text'], 'UTF-8') > 3000) {
		$smarty->append('errors', '本文は3000文字以内で入力してください。');
	}
	if (mb_strlen($_POST['tag'], 'UTF-8') > 200) {
		$smarty->append('errors', 'タグは200文字以内で入力してください。');
	}

	if (is_array($_POST['date'])) {
		$year   = mb_convert_kana($_POST['date']['year'], 'n', 'UTF-8');
		$month  = mb_convert_kana($_POST['date']['month'], 'n', 'UTF-8');
		$day    = mb_convert_kana($_POST['date']['day'], 'n', 'UTF-8');
		$hour   = mb_convert_kana($_POST['date']['hour'], 'n', 'UTF-8');
		$minute = mb_convert_kana($_POST['date']['minute'], 'n', 'UTF-8');

		$_POST['date'] = sprintf('%04d-%02d-%02d %02d:%02d:00', $year, $month, $day, $hour, $minute);
	} else {
		if (!preg_match('/^\d\d\d\d-\d\d-\d\d \d\d:\d\d$/', $_POST['date'])) {
			$smarty->append('errors', '日時が正しく指定されていません。');
		}

		$_POST['date'] = $_POST['date'] . ':00';
	}

	if (isset($_POST['preview']) or $smarty->get_template_vars('errors')) {
		if (isset($_POST['preview']) and !$smarty->get_template_vars('errors') and token_check()) {
			$smarty->assign('article', array(
				'preview' => true,
				'id'      => $_POST['id'] ? $_POST['id'] : 0,
				'stat'    => $_POST['stat'],
				'break'   => $_POST['break'],
				'comt'    => $_POST['comt'],
				'tb'      => $_POST['tb'],
				'date'    => $_POST['date'],
				'subj'    => trim($_POST['subj']),
				'text'    => $_POST['break'] ? '<p>' . nl2br(trim($_POST['text'])) . '</p>' : trim($_POST['text']),
				'tag'     => trim($_POST['tag'])
			));
		}

		admin_article_form();

		exit;
	}

	//プラグイン実行
	plugin('begin');

	//データ投稿
	if (($tag = trim($_POST['tag'])) == '') {
		$tag = null;
	}
	if (empty($_POST['id'])) {
		$stmt = $pdo->prepare('INSERT INTO ' . DATABASE_PREFIX . 'articles VALUES(NULL, :stat, :break, :comt, :tb, :date, :subj, :text, :tag)');
		$stmt->bindValue(':stat',  $_POST['stat'], PDO::PARAM_INT);
		$stmt->bindValue(':break', $_POST['break'], PDO::PARAM_INT);
		$stmt->bindValue(':comt',  $_POST['comt'], PDO::PARAM_INT);
		$stmt->bindValue(':tb',    $_POST['tb'], PDO::PARAM_INT);
		$stmt->bindValue(':date',  $_POST['date']);
		$stmt->bindValue(':subj',  trim($_POST['subj']));
		$stmt->bindValue(':text',  trim($_POST['text']));
		$stmt->bindValue(':tag',   $tag);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$smarty->assign('message', '記事を新規に投稿しました。');
	} else {
		$stmt = $pdo->prepare('UPDATE ' . DATABASE_PREFIX . 'articles SET stat = :stat, break = :break, comt = :comt, tb = :tb, date = :date, subj = :subj, text = :text, tag = :tag WHERE id = :id');
		$stmt->bindValue(':stat',  $_POST['stat'], PDO::PARAM_INT);
		$stmt->bindValue(':break', $_POST['break'], PDO::PARAM_INT);
		$stmt->bindValue(':comt',  $_POST['comt'], PDO::PARAM_INT);
		$stmt->bindValue(':tb',    $_POST['tb'], PDO::PARAM_INT);
		$stmt->bindValue(':date',  $_POST['date']);
		$stmt->bindValue(':subj',  trim($_POST['subj']));
		$stmt->bindValue(':text',  trim($_POST['text']));
		$stmt->bindValue(':tag',   $tag);
		$stmt->bindValue(':id',    $_POST['id'], PDO::PARAM_INT);
		$flag = $stmt->execute();
		if (!$flag) {
			db_error($stmt->errorInfo());
		}

		$smarty->assign('message', 'No.' . $_POST['id'] . 'の記事を編集しました。');
	}

	//トラックバック送信
	if (!empty($_POST['tb_url'])) {
		if (empty($_POST['id'])) {
			$id = $pdo->lastInsertId();
		} else {
			$id = $_POST['id'];
		}

		if (REWRITE_MODE) {
			$url = preg_replace('/\/$/', '', HTTP_URL);
		} else {
			$url = HTTP_URL . MAIN_FILE;
		}

		$excerpt = str_replace("\n", '', strip_tags($_POST['text']));
		$excerpt = strlen($excerpt) > 255 ? mb_strimwidth($excerpt, 0, 252, '...', 'UTF-8') : $excerpt;
		$query   = 'title=' . urlencode($_POST['subj']) . '&url=' . urlencode($url . '/view/' . $id) . '&excerpt=' . urlencode($excerpt) . '&blog_name=' . urlencode($configs['_blog']['title']);

		$errors = array();

		foreach (explode("\n", $_POST['tb_url']) as $tb_url) {
			$info = parse_url($tb_url);

			$request  = "POST " . $info['path'] . (isset($info['query']) ? '?' . $info['query'] : '') . " HTTP/1.0\r\n";
			$request .= "Host: " . $info['host'] . "\r\n";
			$request .= "User-Agent: PHP Blog\r\n";
			$request .= "Content-Type: application/x-www-form-urlencoded\r\n";
			$request .= "Content-Length: " . strlen($query) . "\r\n";
			$request .= "\r\n";
			$request .= "$query\r\n";

			$data = '';

			if ($sock = fsockopen($info['host'], 80)) {
				fputs($sock, $request);
				while (!feof($sock)) {
					$data .= fgets($sock);
				}
				fclose($sock);
			} else {
				$errors[] = 'Socket error : ' . $tb_url;
			}

			if (!preg_match('/<error>0<\/error>/', $data, $matches)) {
				$errors[] = 'Response error from ' . $tb_url;
			} elseif (preg_match('/<error>1<\/error>[^<]*<message>(.+)<\/message>/', $data, $matches)) {
				$errors[] = 'Response error : ' . $matches[1] . ' from ' . $tb_url;
			}
		}

		if (!empty($errors)) {
			$smarty->assign('message', $smarty->get_template_vars('message') . 'トラックバックの送信に失敗しました。' . implode('、', $errors));
		}
	}

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//記事一覧
	admin_article();

	return;
}

/* 管理者用 - 記事削除 */
function admin_article_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_REQUEST['id'])) {
		error('削除したい記事を選択してください。');
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_PREFIX . 'articles WHERE id = :id');
	$stmt->bindValue(':id', $_REQUEST['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_PREFIX . 'comments WHERE article_id = :id');
	$stmt->bindValue(':id', $_REQUEST['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$stmt = $pdo->prepare('DELETE FROM ' . DATABASE_PREFIX . 'trackbacks WHERE article_id = :id');
	$stmt->bindValue(':id', $_REQUEST['id'], PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_PREFIX . 'articles AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_PREFIX . 'comments AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}

		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_PREFIX . 'trackbacks AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . $_REQUEST['id'] . 'の記事を削除しました。');

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//記事一覧
	admin_article();

	return;
}

/* 管理者用 - ファイル一覧 */
function admin_file()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//プラグイン実行
	plugin('begin');

	//アップロードファイル一覧取得
	if ($dir = scandir(FILE_DIR)) {
		$files = array();
		foreach ($dir as $entry) {
			if (!is_file(FILE_DIR . $entry)) {
				continue;
			}

			$size = intval(filesize(FILE_DIR . $entry) / 1024);
			if ($size == 0) {
				$size = 1;
			}

			$files[] = array(
				'name' => $entry,
				'size' => $size,
				'date' => date('Y-m-d H:i:s', filemtime(FILE_DIR . $entry))
			);
		}

		$smarty->assign('files', $files);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_file.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - ファイル追加 */
function admin_file_post()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//プラグイン実行
	plugin('begin');

	//ファイルアップロード
	if (is_uploaded_file($_FILES['file']['tmp_name'])) {
		if (!preg_match('/^[\w\.\~\-\&\#\+\=\;\@\%]+$/', $_FILES['file']['name'])) {
			error('ファイル名は半角英数字で指定してください。');
		}
		if (file_exists(FILE_DIR . $_FILES['file']['name'])) {
			error('ファイル「' . $_FILES['file']['name'] . '」はすでに存在します。');
		}

		if (move_uploaded_file($_FILES['file']['tmp_name'], FILE_DIR . $_FILES['file']['name'])) {
			chmod(FILE_DIR . $_FILES['file']['name'], 0644);
		} else {
			exit('ファイルをアップロードできません。');
		}
	} else {
		error('ファイルを選択してください。');
	}

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//ファイル一覧
	admin_file();

	return;
}

/* 管理者用 - ファイル削除 */
function admin_file_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (!isset($_POST['name'])) {
		error('削除したいファイルを選択してください。');
	}

	//プラグイン実行
	plugin('begin');

	//ファイル削除
	foreach ($_POST['name'] as $name) {
		if (preg_match('/^[\w\-]+\.\w+$/', $name)) {
			unlink(FILE_DIR . $name);
		}
	}

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//ファイル一覧
	admin_file();

	return;
}

/* 管理者用 - コメント一覧 */
function admin_comment()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//表示ページ取得
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//プラグイン実行
	plugin('begin');

	//コメント取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'comments ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$comments = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$data['description'] = strip_tags($data['text']);
		$data['description'] = strlen($data['description']) > 255 ? mb_strimwidth($data['description'], 0, 252, '...', 'UTF-8') : $data['description'];

		$comments[] = $data;
	}
	$smarty->assign('comments', $comments);

	//コメント数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'comments');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data          = $stmt->fetch(PDO::FETCH_NUM);
	$comment_count = $data[0];
	$comment_page  = ceil($comment_count / $configs['_blog']['admin_size']);

	$smarty->assign('comment_count', $comment_count);
	$smarty->assign('comment_page',  $comment_page);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_comment.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - コメント削除 */
function admin_comment_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['id'])) {
		error('削除したいコメントを選択してください。');
	}

	$dels = array();
	foreach ($_POST['id'] as $id) {
		$dels[] = intval($id);
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->query('DELETE FROM ' . DATABASE_PREFIX . 'comments WHERE id IN(' . implode(',', $dels) . ')');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_PREFIX . 'comments AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . implode('、', $dels) . 'のコメントを削除しました。');

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//コメント一覧
	admin_comment();

	return;
}

/* 管理者用 - トラックバック一覧 */
function admin_trackback()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//表示ページ取得
	if (!isset($_GET['page']) or !preg_match('/^\d+$/', $_GET['page']) or $_GET['page'] < 1) {
		$_GET['page'] = 1;
	}

	//プラグイン実行
	plugin('begin');

	//トラックバック取得
	$stmt = $pdo->prepare('SELECT * FROM ' . DATABASE_PREFIX . 'trackbacks ORDER BY id DESC LIMIT :start, :limit');
	$stmt->bindValue(':start', intval($configs['_blog']['admin_size']) * ($_GET['page'] - 1), PDO::PARAM_INT);
	$stmt->bindValue(':limit', intval($configs['_blog']['admin_size']), PDO::PARAM_INT);
	$flag = $stmt->execute();
	if (!$flag) {
		db_error($stmt->errorInfo());
	}

	$trackbacks = array();
	while ($data = $stmt->fetch(PDO::FETCH_ASSOC)) {
		$trackbacks[] = $data;
	}
	$smarty->assign('trackbacks', $trackbacks);

	//トラックバック数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data            = $stmt->fetch(PDO::FETCH_NUM);
	$trackback_count = $data[0];
	$trackback_page  = ceil($trackback_count / $configs['_blog']['admin_size']);

	$smarty->assign('trackback_count', $trackback_count);
	$smarty->assign('trackback_page',  $trackback_page);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_trackback.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - トラックバック削除 */
function admin_trackback_del()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['id'])) {
		error('削除したいトラックバックを選択してください。');
	}

	$dels = array();
	foreach ($_POST['id'] as $id) {
		$dels[] = intval($id);
	}

	//プラグイン実行
	plugin('begin');

	//データ削除
	$stmt = $pdo->query('DELETE FROM ' . DATABASE_PREFIX . 'trackbacks WHERE id IN(' . implode(',', $dels) . ')');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	if (DATABASE_TYPE == 'mysql') {
		$stmt = $pdo->query('ALTER TABLE ' . DATABASE_PREFIX . 'trackbacks AUTO_INCREMENT = 0');
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	$smarty->assign('message', 'No.' . implode('、', $dels) . 'のトラックバックを削除しました。');

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//トラックバック一覧
	admin_trackback();

	return;
}

/* 管理者用 - 設定一覧 */
function admin_config()
{
	global $users, $configs, $params, $pdo, $smarty;

	//ワンタイムトークン作成
	$smarty->assign('token', token_create());

	//作業対象ファイル取得
	if (!isset($_REQUEST['file']) or !preg_match('/^[\w\-]+$/', $_REQUEST['file'])) {
		$_REQUEST['file'] = '_blog';
	}

	//プラグイン実行
	plugin('begin');

	//設定一覧取得
	$files = array(
		array(
			'name'    => '_blog',
			'explain' => '本体'
		)
	);

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (preg_match('/^([^\_]+\w+)\.ini$/', $entry, $matches)) {
				$files[] = array(
					'name'    => $matches[1],
					'explain' => $matches[1] . 'プラグイン'
				);
			}
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	$smarty->assign('files', $files);

	//設定内容取得
	$config_flag = false;
	$buffer      = '';
	$count       = 0;
	$configs     = array();
	$types       = array();
	$sizes       = array();
	$exts        = array();

	if ($fp = fopen(CONFIG_DIR . $_REQUEST['file'] . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if ($buffer and (preg_match('/^\[.+\]$/', $line) or preg_match('/^;.+$/', $line))) {
				$configs[] = array(
					'type' => 'config',
					'name' => null,
					'data' => '<textarea name="data[_buffer' . ++$count . ']" cols="50" rows="5">' . (isset($_POST['config']['data']['_buffer' . $count]) ? $_POST['config']['data']['_buffer' . $count] : $buffer) . '</textarea>'
				);

				$buffer = '';
			}

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$config_flag = true;

				$configs[] = array(
					'type' => 'section',
					'name' => null,
					'data' => $matches[1]
				);
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if ($matches[1] == 'exit') {
					break;
				}

				if ($config_flag) {
					$configs[] = array(
						'type' => 'comment',
						'name' => null,
						'data' => $matches[1]
					);
				} else {
					$define = $matches[1];

					if (preg_match('/^(\S+)\s+(\S+)\s+"(\S+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(\S+)\s+(\S+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(\S+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
						$sizes[$name] = $size;
						$exts[$name]  = $ext;
					}
				}
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$config_flag) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				$value = str_replace('<', '&lt;', $value);
				$value = str_replace('>', '&gt;', $value);

				if (empty($types[$name])) {
					$buffer .= $line . "\n";
				} else {
					$data = '';

					switch ($types[$name]) {
						case 'set':
							$options = explode('|', $exts[$name]);

							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}
							if (isset($_POST['config']['data'][$name])) {
								$value = implode(',', $_POST['config']['data'][$name]);
							}

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if (in_array($option_key, explode(',', $value))) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="checkbox" name="data[' . $name . '][]" id="label_' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="label_' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'enum':
							$options = explode('|', $exts[$name]);

							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$value = $matches[1];
							}
							if (isset($_POST['config']['data'][$name])) {
								$value = $_POST['config']['data'][$name];
							}

							foreach ($options as $option) {
								list($option_key, $option_value) = explode(':', $option, 2);

								if ($value == $option_key) {
									$checked = ' checked="checked"';
								} else {
									$checked = '';
								}

								$data .= '<input type="radio" name="data[' . $name . ']" id="label_' . $name . '_' . $option_key . '" value="' . $option_key . '"' . $checked . ' /> <label for="label_' . $name . '_' . $option_key . '"">' . $option_value . '</label><br />';
							}

							break;
						case 'bool':
							if (empty($exts[$name])) {
								$options = array('On', 'Off');
							} else {
								$options = explode('|', $exts[$name], 2);
							}

							if (isset($_POST['config']['data'][$name])) {
								$value = $_POST['config']['data'][$name];
							}

							$data .= '<select name="data[' . $name . ']">';
							if ($value == 'On') {
								$data .= '<option value="On" selected="selected">' . $options[0] . '</option>';
								$data .= '<option value="Off">' . $options[1] . '</option>';
							} else {
								$data .= '<option value="On">' . $options[0] . '</option>';
								$data .= '<option value="Off" selected="selected">' . $options[1] . '</option>';
							}
							$data .= '</select>';

							break;
						case 'text':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', "\a", $matches[1]);
								$matches[1] = str_replace('\n', "\n", $matches[1]);
								$matches[1] = str_replace("\a", '\\', $matches[1]);

								$value = $matches[1];
							}
							if (isset($_POST['config']['data'][$name])) {
								$value = $_POST['config']['data'][$name];
							}

							$data .= '<textarea name="data[' . $name . ']" cols="50" rows="5">' . $value . '</textarea>';

							break;
						case 'char':
							if (preg_match('/^"(.*)"$/', $value, $matches)) {
								$matches[1] = preg_replace('/" CONFIG_QUOTE "/', '&quot;', $matches[1]);
								$matches[1] = str_replace('\\\\', '\\', $matches[1]);

								$value = $matches[1];
							}
							if (isset($_POST['config']['data'][$name])) {
								$value = $_POST['config']['data'][$name];
							}

							$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 80 : $sizes[$name]) . '" value="' . $value . '" />';

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
						case 'int':
							if (isset($_POST['config']['data'][$name])) {
								$value = $_POST['config']['data'][$name];
							}

							$data .= '<input type="text" name="data[' . $name . ']" size="' . (empty($sizes[$name]) ? 10 : $sizes[$name]) . '" value="' . $value . '" />';

							if (!empty($exts[$name])) {
								$data .= ' ' . $exts[$name];
							}

							break;
					}

					$configs[] = array(
						'type' => 'config',
						'name' => $name,
						'data' => $data
					);
				}
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	$smarty->assign('form', array(
		'file'    => $_REQUEST['file'],
		'configs' => $configs
	));

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_config.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - 設定編集 */
function admin_config_post()
{
	global $users, $configs, $params, $pdo, $smarty;

	//アクセス元チェック
	if (isset($_SERVER['HTTP_REFERER']) and !preg_match('/^' . preg_quote(HTTPS_URL ? HTTPS_URL : HTTP_URL, '/') . '/', $_SERVER['HTTP_REFERER'])) {
		error('不正なアクセスです。');
	}

	//ワンタイムトークン比較
	if (!token_check()) {
		error('不正なアクセスです。');
	}

	//入力内容チェック
	if (empty($_POST['file'])) {
		error('設定対象が指定されていません。');
	}
	if (!preg_match('/^[\w\-]+$/', $_POST['file'])) {
		error('設定対象の指定内容が正しくありません。');
	}

	//プラグイン実行
	plugin('begin');

	//設定編集
	$config_flag = false;
	$buffer_flag = false;
	$data        = '';
	$count       = 0;
	$types       = array();

	if ($fp = fopen(CONFIG_DIR . $_POST['file'] . '.ini', 'r')) {
		while ($line = fgets($fp)) {
			$line = trim($line);

			if (preg_match('/^\[(.+)\]$/', $line, $matches)) {
				$config_flag = true;

				$data .= $line . "\n";
			} elseif (preg_match('/^;(.+)$/', $line, $matches)) {
				if (!$config_flag) {
					$define = $matches[1];

					if (preg_match('/^(\S+)\s+(\S+)\s+"(\S+)"$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = $matches[3];
					} elseif (preg_match('/^(\S+)\s+(\S+)$/', $define, $matches)) {
						$name = $matches[1];
						$type = $matches[2];
						$ext  = '';
					} else {
						$name = '';
						$type = '';
						$ext  = '';
					}

					if ($name) {
						if (preg_match('/^(\S+)\((\d+)\)$/', $type, $matches)) {
							$type = $matches[1];
							$size = $matches[2];
						} else {
							$size = '';
						}

						$types[$name] = $type;
					}
				}
				if ($buffer_flag) {
					$buffer_flag = false;
				}

				$data .= $line . "\n";
			} elseif (preg_match('/^([^\s]+)\s*=\s*(.+)$/', $line, $matches)) {
				if (!$config_flag) {
					continue;
				}

				$name  = $matches[1];
				$value = $matches[2];

				if (empty($types[$name])) {
					if ($buffer_flag) {
						continue;
					}

					if (isset($_POST['data']['_buffer' . ($count + 1)])) {
						$value = trim($_POST['data']['_buffer' . ++$count]);

						$buffer_flag = true;
					} else {
						$value = $line;
					}

					$data .= trim($value) . "\n";
				} else {
					switch ($types[$name]) {
						case 'set':
							if (isset($_POST['data'][$name])) {
								$value = '"' . implode(',', $_POST['data'][$name]) . '"';
							} else {
								$value = '""';
							}
							break;
						case 'enum':
							if (isset($_POST['data'][$name])) {
								$value = '"' . $_POST['data'][$name] . '"';
							}
							break;
						case 'bool':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							break;
						case 'text':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							break;
						case 'char':
							if (isset($_POST['data'][$name])) {
								$value = '"' . preg_replace('/"/', '" CONFIG_QUOTE "', $_POST['data'][$name]) . '"';
							}
							break;
						case 'int':
							if (isset($_POST['data'][$name])) {
								$value = $_POST['data'][$name];
							}
							break;
					}

					$value = str_replace('\\', '\\\\', $value);
					$value = str_replace("\n", '\n', $value);

					$data .= $name . ' = ' . trim($value) . "\n";
				}
			} else {
				$data .= $line . "\n";
			}
		}
		fclose($fp);
	} else {
		exit('設定ファイルを読み込めません。');
	}

	if (!file_put_contents(CONFIG_DIR . $_POST['file'] . '.ini', $data)) {
		exit('設定ファイルに書き込めません。');
	}

	$smarty->assign('message', '設定ファイルを編集しました。');

	//ワンタイムトークン破棄
	token_destroy();

	//プラグイン実行
	plugin('end');

	//設定一覧
	admin_config();

	return;
}

/* 管理者用 - ステータス */
function admin_status()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//記事数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'articles');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['article'] = $count;

	//コメント数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'comments');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['comment'] = $count;

	//トラックバック数取得
	$stmt = $pdo->query('SELECT COUNT(*) FROM ' . DATABASE_PREFIX . 'trackbacks');
	if (!$stmt) {
		db_error($pdo->errorInfo());
	}

	$data  = $stmt->fetch(PDO::FETCH_NUM);
	$count = $data[0];

	$status['trackback'] = $count;

	//アップロードファイルサイズ取得
	if ($dir = scandir(FILE_DIR)) {
		$file = 0;
		foreach ($dir as $entry) {
			if (!is_file(FILE_DIR . $entry)) {
				continue;
			}

			$file += filesize(FILE_DIR . $entry);
		}

		$status['file'] = intval($file / 1024);
	} else {
		exit('アップロードファイル格納ディレクトリを開けません。');
	}

	$smarty->assign('status', $status);

	//プラグイン実行
	plugin('display');

	//データ表示
	output('admin_status.html');

	//プラグイン実行
	plugin('end');

	return;
}

/* 管理者用 - ログアウト */
function admin_logout()
{
	global $users, $configs, $params, $pdo, $smarty;

	//プラグイン実行
	plugin('begin');

	//ログイン情報クリア
	if (isset($_COOKIE['session'])) {
		unlink(LOGIN_DIR . $_COOKIE['session']);

		if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
			$info = parse_url(HTTPS_URL);
		} else {
			$info = parse_url(HTTP_URL);
		}

		setcookie('session', '', 0, $info['path']);
	}
	$_SESSION['login'] = false;

	//プラグイン実行
	plugin('end');

	//管理画面
	admin();

	return;
}

/* エラー表示 */
function error($message)
{
	global $users, $configs, $params, $pdo, $smarty;

	$smarty->assign('message', $message);

	output('error.html');

	exit;
}

/* データベースエラー表示 */
function db_error($info)
{
	global $users, $configs, $params, $pdo, $smarty;

	list($state, $code, $message) = $info;

	exit('Error: ' . $message);
}

/* トラックバック受信エラー表示 */
function tb_error($message)
{
	global $users, $configs, $params, $pdo, $smarty;

	header('Content-Type: text/xml');

	echo "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n";
	echo "<response>\n";
	echo "<error>1</error>\n";
	echo "<message>$message</message>\n";
	echo "</response>\n";

	exit;
}

/* データ出力 */
function output($template, $content = null)
{
	global $users, $configs, $params, $pdo, $smarty;

	if (preg_match('/\.xml$/i', $template)) {
		$content = 'text/xml';
	} elseif (preg_match('/\.txt$/i', $template)) {
		$content = 'text/plain';
	} else {
		$content = $users['content'];
	}

	header('Content-Type: ' . $content . '; charset=UTF-8');

	if ($users['type'] == 'mobile') {
		header('Pragma: no-cache');
		header('Cache-Control: no-cache');

		if ($smarty->template_exists(TEMPLATE_MOBILE . $template)) {
			$template = TEMPLATE_MOBILE . $template;
		}
	}

	if (PICTOGRAM_MODE) {
		echo pictogram_convert($smarty->fetch($template));
	} else {
		$smarty->display($template);
	}

	return;
}

/* タグ整形 */
function create_tag($string)
{
	if (!$string) {
		return;
	}

	if (REWRITE_MODE) {
		$script_url = preg_replace('/\/$/', '', HTTP_URL);
	} else {
		$script_url = HTTP_URL . MAIN_FILE;
	}

	$tags = array();
	foreach (explode(',', $string) as $tag) {
		$tags[] = '<a href="' . $script_url . '/search/tag/' . urlencode($tag) . '">' . htmlspecialchars($tag, ENT_QUOTES) . '</a>';
	}

	return implode(' ', $tags);
}

/* ユーザー情報取得 */
function user()
{
	$users = array();

	//キャリア取得
	if (preg_match('/DoCoMo/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'docomo';
	} elseif (preg_match('/(SoftBank|Vodafone)/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'softbank';
	} elseif (preg_match('/KDDI/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'au';
	} elseif (preg_match('/WILLCOM/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'willcom';
	} elseif (preg_match('/emobile/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['career'] = 'emobile';
	} else {
		$users['career'] = null;
	}

	//接続環境取得
	if ($users['career']) {
		$users['type'] = 'mobile';
	} elseif (preg_match('/Android/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'android';
	} elseif (preg_match('/iPad/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'ipad';
	} elseif (preg_match('/iPhone/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'iphone';
	} elseif (preg_match('/iPod/', $_SERVER['HTTP_USER_AGENT'])) {
		$users['type'] = 'ipod';
	} else {
		$users['type'] = 'pc';
	}

	//コンテンツタイプ取得
	if ($users['career'] == 'au' or $users['type'] != 'mobile') {
		$users['content'] = 'text/html';
	} else {
		$users['content'] = 'application/xhtml+xml';
	}

	//文字コード取得
	if ($users['career'] == 'softbank' or $users['type'] != 'mobile') {
		$users['charset'] = 'utf-8';
	} else {
		$users['charset'] = 'sjis';
	}

	//フォームメソッド取得
	if ($users['career'] == 'au') {
		$users['method'] = 'get';
	} else {
		$users['method'] = 'post';
	}

	//固体識別情報取得
	if (isset($_SERVER['HTTP_X_DCMGUID'])) {
		$users['serial'] = $_SERVER['HTTP_X_DCMGUID'];
	} elseif (isset($_SERVER['HTTP_X_UP_SUBNO'])) {
		$users['serial'] = $_SERVER['HTTP_X_UP_SUBNO'];
	} elseif (isset($_SERVER['HTTP_X_JPHONE_UID'])) {
		$users['serial'] = $_SERVER['HTTP_X_JPHONE_UID'];
	} else {
		$users['serial'] = null;
	}

	return $users;
}

/* セッション開始 */
function session()
{
	if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] != 'off') {
		$info = parse_url(HTTPS_URL);
	} else {
		$info = parse_url(HTTP_URL);
	}

	session_set_cookie_params(SESSION_LIFETIME, $info['path']);
	session_cache_limiter(SESSION_CACHE);
	session_start();
	session_regenerate_id(true);

	return;
}

/* 受信データ正規化 */
function normalize()
{
	global $users;

	//絵文字コード統一
	if (PICTOGRAM_MODE) {
		$_GET  = pictogram_unify($_GET);
		$_POST = pictogram_unify($_POST);
	}

	//不正データ削除
	$_GET     = sanitize($_GET);
	$_POST    = sanitize($_POST);
	$_REQUEST = sanitize($_REQUEST);
	$_SERVER  = sanitize($_SERVER);
	$_COOKIE  = sanitize($_COOKIE);

	//改行コード統一
	$_GET     = unify($_GET);
	$_POST    = unify($_POST);
	$_REQUEST = unify($_REQUEST);
	$_SERVER  = unify($_SERVER);
	$_COOKIE  = unify($_COOKIE);

	//文字コード変換
	if ($users['charset'] == 'sjis') {
		$_GET     = convert($_GET);
		$_POST    = convert($_POST);
		$_REQUEST = convert($_REQUEST);
		$_SERVER  = convert($_SERVER);
	}

	//GETデータ取得
	if ($users['method'] == 'get') {
		$_POST = $_GET;
	}

	return;
}

/* 設定読み込み */
function config()
{
	$configs = array();

	if ($dir = scandir(CONFIG_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(CONFIG_DIR . $entry) or !preg_match('/^(\w+)\.ini$/', $entry, $matches)) {
				continue;
			}

			$config = parse_ini_file(CONFIG_DIR . $entry);

			foreach ($config as $key => $value) {
				$value = str_replace('\\\\', "\a", $value);
				$value = str_replace('\n', "\n", $value);
				$value = str_replace("\a", '\\', $value);

				$configs[$matches[1]][$key] = $value;
			}
		}
	} else {
		exit('設定ファイル格納ディレクトリを開けません。');
	}

	return $configs;
}

/* URLルーティング */
function routing()
{
	$request_uri = explode('/', strtok($_SERVER['REQUEST_URI'], '?'));
	$script_name = explode('/', $_SERVER['SCRIPT_NAME']);

	for ($i = 0; $i < sizeof($script_name); $i++) {
		if ($request_uri[$i] == $script_name[$i]) {
			unset($request_uri[$i]);
		}
	}
	$params = array_values(array_map('urldecode', $request_uri));

	if (isset($params[0]) and preg_match('/\D/', $params[0])) {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = $params[0];
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	} else {
		if (!isset($_REQUEST['mode'])) {
			$_REQUEST['mode'] = '';
		}
		if (!isset($_REQUEST['work'])) {
			$_REQUEST['work'] = '';
		}
	}

	return $params;
}

/* データベース接続 */
function database()
{
	if (DATABASE_TYPE == 'sqlite3') {
		$dsn = 'sqlite:' . DATABASE_NAME;
	} elseif (DATABASE_TYPE == 'sqlite2') {
		$dsn = 'sqlite2:' . DATABASE_NAME;
	} else {
		$dsn = 'mysql:dbname=' . DATABASE_NAME . ';host=' . DATABASE_HOST . (DATABASE_PORT ? ';port=' . DATABASE_PORT : '');
	}

	if (DATABASE_TYPE == 'mysql') {
		$options = array(
			PDO::ATTR_ERRMODE => PDO::ERRMODE_SILENT,
			PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true
		);
	} else {
		$options = array(
			PDO::ATTR_ERRMODE => PDO::ERRMODE_SILENT
		);
	}

	try {
		$pdo = new PDO($dsn, DATABASE_USER, DATABASE_PASSWORD, $options);
	} catch (PDOException $e) {
		exit($e->getMessage());
	}

	if (DATABASE_TYPE == 'mysql' and DATABASE_CHARSET) {
		$stmt = $pdo->query('SET NAMES ' . DATABASE_CHARSET);
		if (!$stmt) {
			db_error($pdo->errorInfo());
		}
	}

	return $pdo;
}

/* テンプレート設定 */
function template()
{
	global $users, $configs;

	$smarty = new Smarty();
	$smarty->template_dir = TEMPLATE_DIR;
	$smarty->compile_dir  = TEMPLATE_COMPILE;

	$smarty->default_modifiers = array('escape');

	if (REWRITE_MODE) {
		$http_file  = preg_replace('/\/$/', '', HTTP_URL);
		$https_file = preg_replace('/\/$/', '', HTTPS_URL ? HTTPS_URL : HTTP_URL);
	} else {
		$http_file  = HTTP_URL . MAIN_FILE;
		$https_file = (HTTPS_URL ? HTTPS_URL : HTTP_URL) . MAIN_FILE;
	}

	if ($users['serial'] and ADMIN_SERIAL and in_array($users['serial'], explode(',', ADMIN_SERIAL))) {
		$login = true;
	} elseif (!empty($_SESSION['login']) or !empty($_COOKIE['session'])) {
		$login = true;
	} else {
		$login = false;
	}

	$smarty->assign('info', array(
		'session_name' => session_name(),
		'session_id'   => session_id(),
		'title'        => $configs['_blog']['title'],
		'description'  => $configs['_blog']['description'],
		'back_url'     => $configs['_blog']['back_url'],
		'configs'      => $configs,
		'http_url'     => HTTP_URL,
		'https_url'    => HTTPS_URL ? HTTPS_URL : HTTP_URL,
		'main_file'    => MAIN_FILE,
		'file_dir'     => FILE_DIR,
		'http_file'    => $http_file,
		'https_file'   => $https_file,
		'login'        => $login,
		'type'         => $users['type'],
		'career'       => $users['career'],
		'content'      => $users['content'],
		'method'       => $users['method'],
		'serial'       => $users['serial']
	));

	return $smarty;
}

/* プラグイン実行 */
function plugin($type)
{
	if ($dir = scandir(PLUGIN_DIR)) {
		foreach ($dir as $entry) {
			if (!is_file(PLUGIN_DIR . $entry) or !preg_match("/^$type\.\w+\.php$/", $entry)) {
				continue;
			}

			include_once PLUGIN_DIR . $entry;
		}
	} else {
		exit('プラグイン格納ディレクトリを開けません。');
	}

	return;
}

/* ワンタイムトークン作成 */
function token_create()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$token = session_id();

	$_SESSION['token'] = $token;

	return $token;
}

/* ワンタイムトークン比較 */
function token_check()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return true;
	}

	if (isset($_REQUEST['token']) and isset($_SESSION['token']) and $_REQUEST['token'] == $_SESSION['token']) {
		return true;
	} else {
		return false;
	}
}

/* ワンタイムトークン破棄 */
function token_destroy()
{
	global $users;

	if ($users['type'] == 'mobile') {
		return;
	}

	$_SESSION['token'] = '';

	return;
}

/* 不正データ削除 */
function sanitize($data)
{
	if (is_array($data)) {
		return array_map('sanitize', $data);
	}

	return str_replace("\0", '', $data);
}

/* 改行コード統一 */
function unify($data)
{
	if (is_array($data)) {
		return array_map('unify', $data);
	}

	$data = preg_replace("/\r?\n/", "\r", $data);
	$data = preg_replace("/\r/", "\n", $data);

	return $data;
}

/* 文字コード変換 */
function convert($data)
{
	if (is_array($data)) {
		return array_map('convert', $data);
	}

	return mb_convert_encoding($data, 'UTF-8', 'UTF-8,SJIS-WIN');
}

/* 絵文字コード統一 */
function pictogram_unify($data)
{
	global $users;
	static $translations;

	if (is_array($data)) {
		return array_map('pictogram_unify', $data);
	}

	if (PICTOGRAM_MODE and $users['type'] == 'pc' and preg_match('/\[e:([^\]]+)\]/', $data)) {
		if (empty($translations)) {

			/* できれば、HTML_Emojiクラスを介さずにDefault.phpへアクセスするのは避けたい */

			$translations = include PICTOGRAM_MAIN_DIR . 'HTML/Emoji/Pc/Default.php';
			$translations = array_flip($translations);
		}

		$data = preg_replace('/\[e:([^\]]+)\]/e', '$translations[pack("H*","$1")]', $data);
	}

	return $data;
}

/* キャリアに応じた絵文字に変換 */
function pictogram_convert($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('pictogram_convert', $data);
	}

	$emoji = HTML_Emoji::getInstance();
	$emoji->setImageUrl(PICTOGRAM_IMAGE_DIR);

	$data = $emoji->convertCarrier($data);

	if ($users['type'] == 'pc') {
		$temporary = md5(uniqid(rand(), true));

		$data = pictogram_escape($data);
		$data = str_replace('$', $temporary, $data);
		$data = preg_replace('/(value="([^"\\\\]|\\\\.)*")/e', 'pictogram_unescape("$1","text")', $data);
		$data = preg_replace('/(<textarea [^>]+>[^<]+<\/textarea>)/e', 'pictogram_unescape("$1","text")', $data);
		$data = str_replace($temporary, '$', $data);
		$data = pictogram_unescape($data);
	}

	return $data;
}

/* 絵文字コード削除 */
function pictogram_except($data)
{
	global $users;

	if (is_array($data)) {
		return array_map('pictogram_except', $data);
	}

	$emoji = HTML_Emoji::getInstance();
	$emoji->setImageUrl(PICTOGRAM_IMAGE_DIR);

	$data = $emoji->removeEmoji($data);

	return $data;
}

/* 絵文字をエスケープ */
function pictogram_escape($data)
{
	return preg_replace('/<img class="emoji" src="([^"]+)" alt="" width="12" height="12" \/>/e', '"[E:".basename("$1",".gif")."]"', $data);
}

/* 絵文字をアンエスケープ */
function pictogram_unescape($data, $type = 'html')
{
	if ($type == 'html') {
		return preg_replace('/\[E:([^\]]+)\]/', '<img class="emoji" src="' . PICTOGRAM_IMAGE_DIR . '$1.gif" alt="" width="12" height="12" />', $data);
	} else {
		return preg_replace('/\[E:([^\]]+)\]/e', '"[e:".basename("$1",".gif")."]"', $data);
	}
}

?>
